@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,6 +34,8 @@
 #include <time.h>
 #include <sys/time.h>
 
+#include <ell/ell.h>
+
 #include "ofono.h"
 
 #include "common.h"
@@ -344,15 +345,15 @@ static char *dbus_apply_text_attributes(const char *text,
 }
 
 static struct stk_menu *stk_menu_create(const char *title,
-		const struct stk_text_attribute *title_attr,
-		const struct stk_icon_id *icon, GSList *items,
-		const struct stk_item_text_attribute_list *item_attrs,
-		const struct stk_item_icon_id_list *item_icon_ids,
-		int default_id, gboolean soft_key, gboolean has_help)
+			const struct stk_text_attribute *title_attr,
+			const struct stk_icon_id *icon, struct l_queue *items,
+			const struct stk_item_text_attribute_list *item_attrs,
+			const struct stk_item_icon_id_list *item_icon_ids,
+			int default_id, bool soft_key, bool has_help)
 {
-	unsigned int len = g_slist_length(items);
+	unsigned int len = l_queue_length(items);
 	struct stk_menu *ret;
-	GSList *l;
+	const struct l_queue_entry *entry;
 	int i;
 	struct stk_text_attribute attr;
 
@@ -364,23 +365,22 @@ static struct stk_menu *stk_menu_create(const char *title,
 	if (item_icon_ids && item_icon_ids->len && item_icon_ids->len != len)
 		return NULL;
 
-	ret = g_try_new(struct stk_menu, 1);
-	if (ret == NULL)
-		return NULL;
+	ret = l_new(struct stk_menu, 1);
 
 	ret->title = dbus_apply_text_attributes(title ? title : "",
 						title_attr);
 	if (ret->title == NULL)
-		ret->title = g_strdup(title ? title : "");
+		ret->title = l_strdup(title ? title : "");
 
 	memcpy(&ret->icon, icon, sizeof(ret->icon));
-	ret->items = g_new0(struct stk_menu_item, len + 1);
+	ret->items = l_new(struct stk_menu_item, len + 1);
 	ret->default_item = -1;
 	ret->soft_key = soft_key;
 	ret->has_help = has_help;
 
-	for (l = items, i = 0; l; l = l->next, i++) {
-		struct stk_item *item = l->data;
+	for (entry = l_queue_get_entries(items), i = 0;
+					entry; entry = entry->next, i++) {
+		struct stk_item *item = entry->data;
 		char *text;
 
 		ret->items[i].item_id = item->id;
@@ -395,7 +395,7 @@ static struct stk_menu *stk_menu_create(const char *title,
 		}
 
 		if (text == NULL)
-			text = strdup(item->text);
+			text = l_strdup(item->text);
 
 		ret->items[i].text = text;
 
@@ -417,8 +417,8 @@ static struct stk_menu *stk_menu_create(const char *title,
 static struct stk_menu *stk_menu_create_from_set_up_menu(
 						const struct stk_command *cmd)
 {
-	gboolean soft_key = (cmd->qualifier & (1 << 0)) != 0;
-	gboolean has_help = (cmd->qualifier & (1 << 7)) != 0;
+	bool soft_key = (cmd->qualifier & (1 << 0)) != 0;
+	bool has_help = (cmd->qualifier & (1 << 7)) != 0;
 
 	return stk_menu_create(cmd->setup_menu.alpha_id,
 				&cmd->setup_menu.text_attr,
@@ -432,8 +432,8 @@ static struct stk_menu *stk_menu_create_from_set_up_menu(
 static struct stk_menu *stk_menu_create_from_select_item(
 						const struct stk_command *cmd)
 {
-	gboolean soft_key = (cmd->qualifier & (1 << 2)) != 0;
-	gboolean has_help = (cmd->qualifier & (1 << 7)) != 0;
+	bool soft_key = (cmd->qualifier & (1 << 2)) != 0;
+	bool has_help = (cmd->qualifier & (1 << 7)) != 0;
 
 	return stk_menu_create(cmd->select_item.alpha_id,
 				&cmd->select_item.text_attr,
@@ -449,11 +449,11 @@ static void stk_menu_free(struct stk_menu *menu)
 	struct stk_menu_item *i;
 
 	for (i = menu->items; i->text; i++)
-		g_free(i->text);
+		l_free(i->text);
 
-	g_free(menu->items);
-	g_free(menu->title);
-	g_free(menu);
+	l_free(menu->items);
+	l_free(menu->title);
+	l_free(menu);
 }
 
 static void emit_menu_changed(struct ofono_stk *stk)
@@ -462,7 +462,7 @@ static void emit_menu_changed(struct ofono_stk *stk)
 	static struct stk_menu no_menu = {
 		.title = "",
 		.items = &end_item,
-		.has_help = FALSE,
+		.has_help = false,
 		.default_item = -1,
 	};
 	static char *name = "MainMenu";
@@ -562,7 +562,7 @@ static gboolean stk_alpha_id_set(struct ofono_stk *stk,
 	else
 		stk_agent_display_action_info(stk->current_agent, alpha, icon);
 
-	g_free(alpha);
+	l_free(alpha);
 
 	return TRUE;
 }
@@ -722,7 +722,7 @@ static DBusMessage *stk_register_agent(DBusConnection *conn,
 					DBUS_TYPE_INVALID) == FALSE)
 		return __ofono_error_invalid_args(msg);
 
-	if (!__ofono_dbus_valid_object_path(agent_path))
+	if (!dbus_validate_path(agent_path, NULL))
 		return __ofono_error_invalid_format(msg);
 
 	stk->default_agent = stk_agent_new(agent_path,
@@ -834,7 +834,7 @@ static DBusMessage *stk_select_item(DBusConnection *conn,
 					DBUS_TYPE_INVALID) == FALSE)
 		return __ofono_error_invalid_args(msg);
 
-	if (!__ofono_dbus_valid_object_path(agent_path))
+	if (!dbus_validate_path(agent_path, NULL))
 		return __ofono_error_invalid_format(msg);
 
 	for (i = 0; i < selection && menu->items[i].text; i++);
@@ -987,7 +987,7 @@ static gboolean handle_command_set_idle_text(const struct stk_command *cmd,
 	}
 
 	if (stk->idle_mode_text)
-		g_free(stk->idle_mode_text);
+		l_free(stk->idle_mode_text);
 
 	if (sim->icon_id.id != 0 && sim->icon_id.qualifier ==
 			STK_ICON_QUALIFIER_TYPE_SELF_EXPLANATORY)
@@ -1041,7 +1041,7 @@ static gboolean timers_cb(gpointer user_data)
 
 static void timer_value_from_seconds(struct stk_timer_value *val, int seconds)
 {
-	val->has_value = TRUE;
+	val->has_value = true;
 	val->hour = seconds / 3600;
 	seconds -= val->hour * 3600;
 	val->minute = seconds / 60;
@@ -1404,7 +1404,7 @@ static gboolean handle_command_display_text(const struct stk_command *cmd,
 	err = stk_agent_display_text(stk->current_agent, text, &dt->icon_id,
 					priority, display_text_cb, stk,
 					display_text_destroy, timeout);
-	g_free(text);
+	l_free(text);
 
 	/* We most likely got an out of memory error, tell SIM to retry */
 	if (err < 0) {
@@ -1443,8 +1443,10 @@ static void set_get_inkey_duration(struct stk_duration *duration,
 	switch (duration->unit) {
 	case STK_DURATION_TYPE_MINUTES:
 		interval = (interval + 59) / 60;
+		break;
 	case STK_DURATION_TYPE_SECONDS:
 		interval = (interval + 9) / 10;
+		break;
 	case STK_DURATION_TYPE_SECOND_TENTHS:
 		break;
 	}
@@ -1453,8 +1455,7 @@ static void set_get_inkey_duration(struct stk_duration *duration,
 }
 
 static void request_confirmation_cb(enum stk_agent_result result,
-					gboolean confirm,
-					void *user_data)
+					bool confirm, void *user_data)
 {
 	struct ofono_stk *stk = user_data;
 	static struct ofono_error error = { .type = OFONO_ERROR_TYPE_FAILURE };
@@ -1467,7 +1468,7 @@ static void request_confirmation_cb(enum stk_agent_result result,
 
 		rsp.result.type = STK_RESULT_TYPE_SUCCESS;
 		rsp.get_inkey.text.text = confirm ? "" : NULL;
-		rsp.get_inkey.text.yesno = TRUE;
+		rsp.get_inkey.text.yesno = true;
 
 		if (cmd->duration.interval) {
 			rsp.get_inkey.duration.unit = cmd->duration.unit;
@@ -1609,7 +1610,7 @@ static gboolean handle_command_get_inkey(const struct stk_command *cmd,
 						&gi->icon_id, request_key_cb,
 						stk, NULL, timeout);
 
-	g_free(text);
+	l_free(text);
 
 	if (err < 0) {
 		unsigned char no_cause_result[] = { 0x00 };
@@ -1703,7 +1704,7 @@ static gboolean handle_command_get_input(const struct stk_command *cmd,
 						request_string_cb,
 						stk, NULL, timeout);
 
-	g_free(text);
+	l_free(text);
 
 	if (err < 0) {
 		unsigned char no_cause_result[] = { 0x00 };
@@ -1760,7 +1761,7 @@ static void call_setup_cancel(struct ofono_stk *stk)
 		__ofono_voicecall_dial_cancel(vc);
 }
 
-static void confirm_call_cb(enum stk_agent_result result, gboolean confirm,
+static void confirm_call_cb(enum stk_agent_result result, bool confirm,
 				void *user_data)
 {
 	struct ofono_stk *stk = user_data;
@@ -1778,7 +1779,7 @@ static void confirm_call_cb(enum stk_agent_result result, gboolean confirm,
 
 	switch (result) {
 	case STK_AGENT_RESULT_TIMEOUT:
-		confirm = FALSE;
+		confirm = false;
 		/* Fall through */
 
 	case STK_AGENT_RESULT_OK:
@@ -1828,7 +1829,7 @@ static void confirm_call_cb(enum stk_agent_result result, gboolean confirm,
 					alpha_id, sc->icon_id_call_setup.id,
 					qualifier >> 1, call_setup_connected,
 					stk);
-	g_free(alpha_id);
+	l_free(alpha_id);
 
 	if (err >= 0) {
 		stk->cancel_cmd = call_setup_cancel;
@@ -1864,7 +1865,7 @@ static void confirm_call_cb(enum stk_agent_result result, gboolean confirm,
 }
 
 static void confirm_handled_call_cb(enum stk_agent_result result,
-					gboolean confirm, void *user_data)
+					bool confirm, void *user_data)
 {
 	struct ofono_stk *stk = user_data;
 	const struct stk_command_setup_call *sc =
@@ -1960,7 +1961,7 @@ static gboolean handle_command_set_up_call(const struct stk_command *cmd,
 	err = stk_agent_confirm_call(stk->current_agent, alpha_id,
 					&sc->icon_id_usr_cfm, confirm_call_cb,
 					stk, NULL, stk->timeout * 1000);
-	g_free(alpha_id);
+	l_free(alpha_id);
 
 	if (err < 0) {
 		unsigned char no_cause_result[] = { 0x00 };
@@ -2019,7 +2020,7 @@ static void send_ussd_callback(int error, int dcs, const unsigned char *msg,
 			rsp.result.type = STK_RESULT_TYPE_SUCCESS;
 			rsp.send_ussd.text.text = msg;
 			rsp.send_ussd.text.len = msg_len;
-			rsp.send_ussd.text.has_text = TRUE;
+			rsp.send_ussd.text.has_text = true;
 		} else
 			rsp.result.type = STK_RESULT_TYPE_USSD_RETURN_ERROR;
 
@@ -2147,7 +2148,7 @@ static gboolean handle_command_send_ussd(const struct stk_command *cmd,
 
 static void free_idle_mode_text(struct ofono_stk *stk)
 {
-	g_free(stk->idle_mode_text);
+	l_free(stk->idle_mode_text);
 	stk->idle_mode_text = NULL;
 
 	memset(&stk->idle_mode_icon, 0, sizeof(stk->idle_mode_icon));
@@ -2163,7 +2164,7 @@ static gboolean handle_command_refresh(const struct stk_command *cmd,
 	struct ofono_sim *sim;
 	uint8_t addnl_info[1];
 	int err;
-	GSList *l;
+	const struct l_queue_entry *entry;
 
 	DBG("");
 
@@ -2207,8 +2208,9 @@ static gboolean handle_command_refresh(const struct stk_command *cmd,
 	}
 
 	DBG("Files:");
-	for (l = cmd->refresh.file_list; l; l = l->next) {
-		struct stk_file *file = l->data;
+	for (entry = l_queue_get_entries(cmd->refresh.file_list);
+					entry; entry = entry->next) {
+		struct stk_file *file = entry->data;
 		char buf[17];
 
 		encode_hex_own_buf(file->file, file->len, 0, buf);
@@ -2272,7 +2274,8 @@ static gboolean handle_command_refresh(const struct stk_command *cmd,
 	 */
 	if (cmd->qualifier < 4 || rsp == NULL) {
 		int qualifier = stk->pending_cmd->qualifier;
-		GSList *file_list = stk->pending_cmd->refresh.file_list;
+		struct l_queue *file_list =
+					stk->pending_cmd->refresh.file_list;
 
 		/* Don't free the list yet */
 		stk->pending_cmd->refresh.file_list = NULL;
@@ -2315,7 +2318,7 @@ static gboolean handle_command_refresh(const struct stk_command *cmd,
 			break;
 		}
 
-		g_slist_free_full(file_list, g_free);
+		l_queue_destroy(file_list, l_free);
 
 		return FALSE;
 	}
@@ -2345,7 +2348,7 @@ static void get_time(struct stk_response *rsp)
 	rsp->provide_local_info.datetime.minute = t->tm_min;
 	rsp->provide_local_info.datetime.second = t->tm_sec;
 	rsp->provide_local_info.datetime.timezone = t->tm_gmtoff / 900;
-	rsp->provide_local_info.datetime.has_timezone = TRUE;
+	rsp->provide_local_info.datetime.has_timezone = true;
 
 	return;
 }
@@ -2615,7 +2618,7 @@ static gboolean handle_command_play_tone(const struct stk_command *cmd,
 	int timeout;
 	int err;
 
-	if (pt->tone > sizeof(tone_infos) / sizeof(*tone_infos) ||
+	if (pt->tone >= L_ARRAY_SIZE(tone_infos) ||
 			tone_infos[pt->tone].name == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
 
@@ -2661,7 +2664,7 @@ static gboolean handle_command_play_tone(const struct stk_command *cmd,
 						play_tone_cb, stk, NULL,
 						timeout);
 
-	g_free(text);
+	l_free(text);
 
 	if (err < 0) {
 		unsigned char no_cause_result[] = { 0x00 };
@@ -2682,7 +2685,7 @@ static gboolean handle_command_play_tone(const struct stk_command *cmd,
 }
 
 static void confirm_launch_browser_cb(enum stk_agent_result result,
-					gboolean confirm,
+					bool confirm,
 					void *user_data)
 {
 	struct ofono_stk *stk = user_data;
@@ -2692,7 +2695,7 @@ static void confirm_launch_browser_cb(enum stk_agent_result result,
 
 	switch (result) {
 	case STK_AGENT_RESULT_TIMEOUT:
-		confirm = FALSE;
+		confirm = false;
 		/* Fall through */
 
 	case STK_AGENT_RESULT_OK:
@@ -2733,7 +2736,7 @@ static gboolean handle_command_launch_browser(const struct stk_command *cmd,
 						lb->icon_id.id, lb->url,
 						confirm_launch_browser_cb,
 						stk, NULL, stk->timeout * 1000);
-	g_free(alpha_id);
+	l_free(alpha_id);
 
 	if (err < 0) {
 		unsigned char no_cause_result[] = { 0x00 };
@@ -2784,7 +2787,7 @@ static gboolean handle_setup_call_confirmation_req(struct stk_command *cmd,
 					confirm_handled_call_cb,
 					stk, NULL,
 					stk->timeout * 1000);
-	g_free(alpha_id);
+	l_free(alpha_id);
 
 	if (err < 0)
 		goto out;
@@ -3149,7 +3152,7 @@ static void stk_unregister(struct ofono_atom *atom)
 		stk->cancel_cmd = NULL;
 	}
 
-	g_free(stk->idle_mode_text);
+	l_free(stk->idle_mode_text);
 	stk->idle_mode_text = NULL;
 
 	if (stk->timers_source) {
